function Figure() {

    this.side = 0;

    this.hitAngles = [ 0, 0.25 * PI, 0.5 * PI, 0.75 * PI, PI, 1.25 * PI, 1.5 * PI, 1.75 * PI ];
    this.halfHitAngle = 0.3 * PI;

    this.headName = "";
    this.head = null;

    this.bodyName = "";
    this.body = null;

    this.armsName = "";
    this.arms = null;

    this.legsName = "";
    this.legs = null;

    this.bodyBob = 0.0;

    this.mainAttackCountdown = 0.0;
    this.secondAttackCountdown = 0.0;

    this.attackAni = 0.0;
    this.hitAni = 0.0;

    this.bangAni = 0.0;
    this.bangPosUpdate = false;
    this.bangX = 0.0;
    this.bangY = 0.0;
    this.bangDir = 0;
    this.bangSize = 0.0;



    this.init = function(side, headName, bodyName, armsName, legsName) {
        this.side = side;
        this.setPart("head", headName);
        this.setPart("body", bodyName);
        this.setPart("arms", armsName);
        this.setPart("legs", legsName);
    };


    this.mainAttack = function(x, y, dir) {
        this.mainAttackCountdown = this.arms.attackLength;
        this.bangAni = 1.0;
        this.bangDir = dir;
        this.bangPosUpdate = true;
        this.bangSize = this.arms.hitSize;

        var updateX = Math.floor(x / map.MOB_UPDATE_GRID_TILE_W);
        var startX = limit(updateX - 1, 0, map.MOB_UPDATE_GRID_W - 1);
        var endX = limit(updateX + 1, 0, map.MOB_UPDATE_GRID_W - 1);

        var updateY = Math.floor(y / map.MOB_UPDATE_GRID_TILE_H);
        var startY = limit(updateY - 1, 0, map.MOB_UPDATE_GRID_H - 1);
        var endY = limit(updateY + 1, 0, map.MOB_UPDATE_GRID_H - 1);

        for(var hitX = startX; hitX <= endX; hitX++) {
            for(var hitY = startY; hitY <= endY; hitY++) {
                var ts = map.mobUpdateGrid[hitX][hitY];
                for(var i = 0; i < ts.length; i++) {
                    var t;
                    if(ts[i] > 1) {
                        t = map.mobs[ts[i]];
                    } else {
                        t = player;
                    }
                    if(t.side != this.side && !t.dead) {
                        var deltaX = t.x - x;
                        var deltaY = t.y - y;
                        if(Math.sqrt((deltaX * deltaX) + (deltaY * deltaY * 0.64)) < this.arms.attackRange) {
                            var angle = Math.atan2(deltaX, -deltaY);
                            var hitAngle = this.hitAngles[dir];
                            if(Math.abs(angleDelta(angle, hitAngle)) <= this.halfHitAngle) {
                                t.hit(this.arms.attackStrength, x, y);
                                if(this.arms.hasOwnProperty("dot")) {
                                    t.dotCountdown = this.arms.dot[0];
                                    t.dotDamage = this.arms.dot[1];
                                }
                                if(this.arms.hasOwnProperty("stun")) {
                                    t.stunCountdown = this.arms.stun;
                                }
                                if(this.arms.hasOwnProperty("knockback")) {
                                    t.knockback(angle, this.arms.knockback);
                                }
                            }
                        }
                    }
                }
            }
        }
    };


    this.secondAttack = function(x, y, dir) {
        /*this.secondAttackCountdown = this.arms.second[1];

        var tileX = Math.floor(x / map.MW);
        var startX = limit(tileX - 4, 0, map.W - 1);
        var endX = limit(tileX + 4, 0, map.W - 1);

        var tileY = Math.floor(y / map.MOB_UPDATE_GRID_TILE_H);
        var startY = limit(tileY - 4, 0, map.H - 1);
        var endY = limit(tileY + 4, 0, map.H - 1);

        for(var hitX = startX; hitX <= endX; hitX++) {
            for(var hitY = startY; hitY <= endY; hitY++) {
                var ts = map.mobUpdateGrid[hitX][hitY];
                for(var i = 0; i < ts.length; i++) {
                    var t;
                    if(ts[i] > 1) {
                        t = map.mobs[ts[i]];
                    } else {
                        t = player;
                    }
                    if(t.side != this.side && !t.dead) {
                        var deltaX = t.x - x;
                        var deltaY = t.y - y;
                        if(Math.sqrt((deltaX * deltaX) + (deltaY * deltaY * 0.64)) < this.arms.attackRange) {
                            var angle = Math.atan2(deltaX, -deltaY);
                            var hitAngle = this.hitAngles[dir];
                            if(Math.abs(angleDelta(angle, hitAngle)) <= this.halfHitAngle) {
                                t.hit(this.arms.attackStrength, x, y);
                                if(this.arms.hasOwnProperty("dot")) {
                                    t.dotCountdown = this.arms.dot[0];
                                    t.dotDamage = this.arms.dot[1];
                                }
                                if(this.arms.hasOwnProperty("stun")) {
                                    t.stunCountdown = this.arms.stun;
                                }
                                if(this.arms.hasOwnProperty("knockback")) {
                                    t.knockback(angle, this.arms.knockback);
                                }
                            }
                        }
                    }
                }
            }
        }*/
    };


    this.canMainAttack = function() {
        return this.mainAttackCountdown <= 0.0;
    };


    this.hasSecondAttack = function() {
        return this.head.hasOwnProperty("second");
    };


    this.canSecondAttack = function() {
        return this.secondAttackCountdown <= 0.0;
    };


    this.hit = function(x, y) {
        this.hitAni = 1.0;
    };


    this.setPart = function(part, name) {
        if(part == "head") {
            this.headName = name;
            this.head = BodyParts.heads[name];
            this.secondAttackCountdown = 0.0;
        } else if(part == "body") {
            this.bodyName = name;
            this.body = BodyParts.bodies[name];
        } else if(part == "arms") {
            this.armsName = name;
            this.arms = BodyParts.arms[name];
            this.mainAttackCountdown = 0.0;
            this.bangAni = 0.0;
        } else if(part == "legs") {
            this.legsName = name;
            this.legs = BodyParts.legs[name];
        }
    };


    this.drawLegs = function(direction, walkAni, WalkAmp) {

        walkAni *= TWO_PI;
        var legAni = Math.sin(walkAni) * 0.8 * this.legs.height * WalkAmp;
        this.bodyBob = Math.round(Math.abs(Math.sin(walkAni - 0.4)) * 0.4 * this.legs.height * WalkAmp);

        var hor;

        // Shadow

        var shadowSize = this.body.shadowSize;
        if(shadowSize < this.legs.shadowSize) {
            shadowSize = this.legs.shadowSize
        }
        img.drawSprite("mobShadow", -22, -14, 48, 32, shadowSize - 1, 0);

        // Legs

        c.translate(0, -this.bodyBob);

        if(direction < 4) {
            hor = this.hor(direction, this.legs.x, this.legs.d + legAni);
            img.drawSprite("legs", -21 + hor.x, -29 + hor.y, 40, 40, direction, this.legs.offset);
        }
        hor = this.hor(direction, -this.legs.x, this.legs.d - legAni);
        img.drawSprite("legs", -21 + hor.x, -29 + hor.y, 40, 40, direction, this.legs.offset);
        if(direction >= 4) {
            hor = this.hor(direction, this.legs.x, this.legs.d + legAni);
            img.drawSprite("legs", -21 + hor.x, -29 + hor.y, 40, 40, direction, this.legs.offset);
        }
    };


    this.update = function(stunCountdown) {
        if(this.mainAttackCountdown > 0.0) {
            if(stunCountdown > 0.0) {
                this.mainAttackCountdown -= 0.6 * timer.delta;
            } else {
                this.mainAttackCountdown -= timer.delta;
            }
            if(this.mainAttackCountdown < 0.0) {
                this.mainAttackCountdown = 0.0;
            }
            var attackLength = this.arms.attackLength;
            var multiplier = 1.0;
            if(attackLength < 0.5) {
                attackLength = 0.5;
                multiplier = 0.5 / this.arms.attackLength;
            }
            this.attackAni = multiplier * this.mainAttackCountdown / attackLength;

        } else if(this.attackAni > 0.0) {
            this.attackAni -= timer.delta * 2.0;
        }
        if(this.bangAni > 0.0) {
            this.bangAni -= timer.delta * 5.0;
            if(this.bangAni < 0.0) {
                this.bangAni = 0.0;
            }
        }
        if(this.hitAni > 0.0) {
            this.hitAni -= timer.delta * 2.0;
            if(this.hitAni < 0.0) {
                this.hitAni = 0.0;
            }
        }
    };


    this.draw = function(direction, walkAni, WalkAmp, deathAnimation) {

        var bangPosLx = 0;
        var bangPosLy = 0;
        var bangPosRx = 0;
        var bangPosRy = 0;


        var lifeStrength = 1.0;
        if(deathAnimation > 0.0) {
            deathAnimation = 1.25 * limit(deathAnimation, 0.0, 0.8);
            deathAnimation = Math.cos(deathAnimation * HALF_PI);
            deathAnimation = 1.0 - deathAnimation;
            deathAnimation = deathAnimation * deathAnimation;
            lifeStrength = 1.0 - deathAnimation;
        }

        var attack = this.attackAni;
        var hit = this.hitAni;

        walkAni *= TWO_PI;
        var armAni = Math.sin(walkAni  - 0.4) * 0.5 * (this.body.height - this.body.armsH) * WalkAmp;
        var bodyHeightForBob = limit(this.body.height, 0, 16);
        var headBob = Math.round(Math.abs(Math.sin(walkAni - 0.8)) * 0.1 * bodyHeightForBob * WalkAmp);

        var armAttackAniR = 0.0;
        var armAttackAniRh = 0.0;
        var armAttackAniRx = 1.0;
        var armAttackAniL = 0.0;
        var armAttackAniLh = 0.0;
        var armAttackAniLx = 1.0;
        var bodyAttackAni = 0.0;
        if(attack > 0.0) {
            attack = 0.5 * (1.0 + Math.cos((1.0 - attack) * PI));
            armAttackAniR = attack * 1.5 * (this.body.height - this.body.armsH);
            armAttackAniRh = attack * -0.8 * (this.body.height - this.body.armsH);
            armAttackAniRx = 1.0 - attack * 0.5;
            bodyAttackAni = attack * 0.3 * this.body.height;
            if(this.arms.oneArmAttack) {
                armAttackAniL = -0.2 * armAttackAniR;
            } else {
                armAttackAniL = armAttackAniR;
                armAttackAniLh = armAttackAniRh;
                armAttackAniLx = armAttackAniRx;
            }
        }

        var hitSquish = 1.0 - (hit * 0.5);
        var hitKnock = hit * -0.15 * this.body.height;

        var hor;

        // Body & Arms

        var bodyOffset = -this.bodyBob + Math.round(-this.legs.height * hitSquish);
        bodyOffset *= lifeStrength;
        c.translate(0, bodyOffset);
        if(deathAnimation > 0.0) {
            if(direction >= 4) {
                c.rotate(deathAnimation * 1.5);
            } else {
                c.rotate(deathAnimation * -1.5);
            }
        }

        if(direction <= 3) {
            hor = this.hor(direction, this.body.armsX * armAttackAniLx, this.body.armsD + armAttackAniL - armAni);
            img.drawSprite("arms", -21 + hor.x, -19 + hor.y + armAttackAniLh - this.body.armsH, 40, 40, direction, this.arms.offset);
            bangPosLx = hor.x;
            bangPosLy = hor.y + armAttackAniLh - this.body.armsH;
        }
        if(direction >= 5 || direction <= 0) {
            hor = this.hor(direction, -this.body.armsX * armAttackAniRx, this.body.armsD + armAttackAniR + armAni);
            img.drawSprite("arms", -21 + hor.x, -19 + hor.y + armAttackAniRh - this.body.armsH, 40, 40, direction, this.arms.offset);
            bangPosRx = hor.x;
            bangPosRy = hor.y + armAttackAniRh - this.body.armsH;
        }

        hor = this.hor(direction, 0, bodyAttackAni + hitKnock);
        img.drawSprite("bodies", -40 + hor.x, -60 + hor.y, 80, 80, direction, this.body.offset);

        if(direction > 3) {
            hor = this.hor(direction, this.body.armsX * armAttackAniLx, this.body.armsD + armAttackAniL - armAni);
            img.drawSprite("arms", -21 + hor.x, -19 + hor.y + armAttackAniLh - this.body.armsH, 40, 40, direction, this.arms.offset);
            bangPosLx = hor.x;
            bangPosLy = hor.y + armAttackAniLh - this.body.armsH;
        }
        if(direction < 5 && direction > 0) {
            hor = this.hor(direction, -this.body.armsX * armAttackAniRx, this.body.armsD + armAttackAniR + armAni);
            img.drawSprite("arms", -21 + hor.x, -19 + hor.y + armAttackAniRh - this.body.armsH, 40, 40, direction, this.arms.offset);
            bangPosRx = hor.x;
            bangPosRy = hor.y + armAttackAniRh - this.body.armsH;
        }

        // Bang

        if(this.bangPosUpdate) {
            if(this.arms.oneArmAttack) {
                this.bangX = bangPosRx;
                this.bangY = bangPosRy;
            } else {
                this.bangX = 0.5 * (bangPosRx + bangPosLx);
                this.bangY = 0.5 * (bangPosRy + bangPosLy);
            }
            this.bangPosUpdate = false;
        }
        if(this.bangAni > 0.0) {
            var bangR = 0.8 * (this.body.height - this.body.armsH) * (1.0 - this.bangAni);
            if(this.armsName == "lindworm") {
                bangR += 24;
            }
            c.globalAlpha = 0.5 * this.bangAni;
            c.fillStyle = "#fff";
            var bangCx = bangR * Figure.HOR_O[this.bangDir];
            var bangCy = (bangR * Figure.HOR_M[this.bangDir]) * -0.7;
            drawCircle(c, this.bangX + bangCx, this.bangY + bangCy, this.bangSize * (2.0 - this.bangAni));
            c.fill();
            c.globalAlpha = 1.0;
        }

        // Head

        c.translate(0, (-this.body.height - headBob) * lifeStrength);
        if(deathAnimation > 0.0) {
            if(direction >= 4) {
                c.rotate(deathAnimation * -3.5);
            } else {
                c.rotate(deathAnimation * 3.5);
            }
        }
        hor = this.hor(direction, this.body.headX, this.body.headD + (1.6 * (bodyAttackAni + hitKnock)));
        img.drawSprite("heads", -20 + hor.x, -40 + hor.y, 40, 50, direction, this.head.offset);

    };


    this.drawStatus = function(isMob, life, maxLife, dot, stun) {
        c.translate(0, -(this.legs.height + this.body.height + this.head.height + 10));

        if(life > 0.0 && dot > 0.0) {
            img.drawSprite("effects", -10, -10, 20, 20, Math.round(dot * 10) % 10, 1);
        }
        if(life > 0.0 && stun > 0.0) {
            img.drawSprite("effects", -10, -10, 20, 20, Math.round(stun * 10) % 10, 0);
        }
        if(isMob && life > 0.0 && life < maxLife) {
            c.fillStyle = "rgba(0, 0, 0, 0.4)";
            c.fillRect(-16, -2, 32, 4);
            c.fillStyle = "#f00";
            c.fillRect(-16, -2, Math.ceil(32 * (life / maxLife)), 4);
        }
    };


    this.hor = function(dir, horX, horD) {
        var x = -horX * Figure.HOR_M[dir];
        x += horD * Figure.HOR_O[dir];
        var y = horD * Figure.HOR_M[dir]
        y += horX * Figure.HOR_O[dir];
        y *= -0.6;
        return { x : Math.round(x), y : Math.round(y) };
    };


    this.getSpeed = function() {
        return this.legs.speed;
    };


    this.getLife = function() {
        var life = 0;
        if(this.head.hasOwnProperty("life")) {
            life += this.head.life;
        }
        if(this.body.hasOwnProperty("life")) {
            life += this.body.life;
        }
        if(this.arms.hasOwnProperty("life")) {
            life += this.arms.life;
        }
        if(this.legs.hasOwnProperty("life")) {
            life += this.legs.life;
        }
        return life;
    };


    this.getDefence = function() {
        var defence = 0;
        if(this.head.hasOwnProperty("defence")) {
            defence += this.head.defence;
        }
        if(this.body.hasOwnProperty("defence")) {
            defence += this.body.defence;
        }
        if(this.arms.hasOwnProperty("defence")) {
            defence += this.arms.defence;
        }
        if(this.legs.hasOwnProperty("defence")) {
            defence += this.legs.defence;
        }
        return defence;
    };

}


Figure.HOR_M = [ 1.0, 0.7, 0.0, -0.7, -1.0, -0.7, 0.0, 0.7 ];

Figure.HOR_O = [ 0.0, 0.7, 1.0, 0.7, 0.0, -0.7, -1.0, -0.7 ];