function Map() {

    this.W = 320;
    this.H = 100;

    this.TILE_W = 44;
    this.TILE_H = 28;

    this.TILE_VAR_W = 22;
    this.TILE_VAR_H = 10;

    this.TILE_RATIO = this.TILE_VAR_W / this.TILE_VAR_H;

    this.TILE_MID_X = 20;
    this.TILE_MID_Y = 15;

    this.MOB_GRID_W = this.W * 4;
    this.MOB_GRID_H = this.H * 4;
    this.MOB_GRID_TILE_W = this.TILE_W / 4;
    this.MOB_GRID_TILE_H = this.TILE_H / 4;
    this.MOB_GRID_OFFSET_X = this.MOB_GRID_TILE_W / 2;
    this.MOB_GRID_OFFSET_Y = -2;

    this.MOB_UPDATE_GRID_W = this.W / 4;
    this.MOB_UPDATE_GRID_H = this.H / 4;
    this.MOB_UPDATE_GRID_TILE_W = Math.ceil((this.W * this.TILE_W) / this.MOB_UPDATE_GRID_W);
    this.MOB_UPDATE_GRID_TILE_H = Math.ceil((this.H * this.TILE_H) / this.MOB_UPDATE_GRID_H);

    this.t;

    this.mobs;
    this.mobCounter;
    this.mobGrid;
    this.mobUpdateGrid;
    this.mobDrawGrid;

    this.spawners;
    this.spawnerCounter;
    this.spawnerUpdateIndex;

    this.updateMapCountdown = 2;

    this.liquidAni = 0.0;


    this.init = function() {
        this.t = createMatrix(this.W, this.H, {});

        for(var x = 0; x < this.W; x++) {
            for(var y = 0; y < this.H; y++) {
                this.t[x][y] = {
                    tileX : 0,
                    tileY : 0,
                    elevation : 0,
                    mobs : [],
                    border : 0,
                    seen : false,
                    liquid : 0,
                    liquidAniOffset : randFloat(0.0, 1.0),
                    path : 0,
                    object : -1,
                };
            }
        }

        this.mobGrid = createMatrix(this.MOB_GRID_W, this.MOB_GRID_H, 0);
        this.mobUpdateGrid = createMatrix(this.MOB_UPDATE_GRID_W, this.MOB_UPDATE_GRID_H, null);
        for(var x = 0; x < this.MOB_UPDATE_GRID_W; x++) {
            for(var y = 0; y < this.MOB_UPDATE_GRID_H; y++) {
                this.mobUpdateGrid[x][y] = [];
            }
        }
        this.mobDrawGrid = createMatrix(this.H, 70, null);
        for(var y = 0; y < this.H; y++) {
            for(var slot = 0; slot < 70; slot++) {
                this.mobDrawGrid[y][slot] = [];
            }
        }

        this.mobs = {};
        this.mobCounter = 2;

        this.spawners = [];
        this.spawnerCounter = 0;
        this.spawnerUpdateIndex = 0;

        MapGenerator.generate();

        this.updateBorders();
        this.checkBlockedTiles();

        MapGenerator.placeSpawners();

        this.startSpawn();
    };


    this.updateBorders = function() {
        for(var x = 0; x < this.W; x++) {
            for(var y = 0; y < this.H; y++) {
                var h = this.t[x][y].elevation;
                for(var i = 0; i < 6; i++) {
                    var neighbor = this.getTileNeighbor(x, y, i);
                    if(this.getTileElevation(neighbor) < h ||
                        this.t[neighbor.x][neighbor.y].liquid != 0) {
                        this.t[x][y].border += Math.pow(2, i);
                    }
                }
            }
        }
    };


    this.addMob = function(type, x, y, side, parentSpawner) {
        var newMob = new Mob();
        newMob.init(type, this.mobCounter, x, y, side, parentSpawner);
        this.mobs[this.mobCounter] = newMob;
        this.mobCounter++;
    };


    this.addSpawner = function(tileX, tileY, type, radius, maxMobs) {
        var pos = this.getTilePos(tileX, tileY);
        var newSpawner = new Spawner();
        newSpawner.init(pos.x, pos.y, type, this.spawnerCounter, radius, maxMobs);
        this.spawners[this.spawnerCounter] = newSpawner;
        this.spawnerCounter++;
    };


    this.getTileNeighbor = function(x, y, dir) {
        var neighbor = { x : x, y : y };
        if(dir == 0) {
            neighbor.y--;
        } else if(dir == 1) {
            neighbor.y--;
            neighbor.x++;
        } else if(dir == 2) {
            neighbor.x++;
        } else if(dir == 3) {
            neighbor.y++;
            neighbor.x++;
        } else if(dir == 4) {
            neighbor.y++;
        } else if(dir == 5) {
            neighbor.x--;
        }
        if(y % 2 == 0 && dir != 2 && dir != 5) {
            neighbor.x--;
        }
        neighbor.x = limit(neighbor.x, 0, this.W - 1);
        neighbor.y = limit(neighbor.y, 0, this.H - 1);
        return neighbor;
    };


    this.checkBlockedTiles = function() {
        var x1;
        var x2;
        var y1;
        var y2;
        var tile1;
        var tile2;
        var tile3;
        var tile4;
        var h1;
        var h2;
        var h3;
        var h4;
        var l1;
        var l2;
        var l3;
        var l4;
        for(var x = 0; x < this.MOB_GRID_W; x++) {
            for(var y = 0; y < this.MOB_GRID_H; y++) {
                x1 = (x * this.MOB_GRID_TILE_W) + this.MOB_GRID_OFFSET_X;
                x2 = x1 + this.MOB_GRID_TILE_W;
                y1 = (y * this.MOB_GRID_TILE_H) + this.MOB_GRID_OFFSET_Y;
                y2 = y1 + this.MOB_GRID_TILE_H;
                tile1 = this.coordToTile(x1, y1);
                tile2 = this.coordToTile(x1, y2);
                tile3 = this.coordToTile(x2, y2);
                tile4 = this.coordToTile(x2, y1);
                h1 = this.getTileElevation(tile1);
                h2 = this.getTileElevation(tile2);
                h3 = this.getTileElevation(tile3);
                h4 = this.getTileElevation(tile4);
                l1 = this.t[tile1.x][tile1.y].liquid != 0;
                l2 = this.t[tile2.x][tile2.y].liquid != 0;
                l3 = this.t[tile3.x][tile3.y].liquid != 0;
                l4 = this.t[tile4.x][tile4.y].liquid != 0;
                if(h1 >= 5 || h2 >= 5 || h3 >= 5 || h4 >= 5) {
                    this.mobGrid[x][y] = -1;
                } else if(l1 || l2 || l3 || l4) {
                    this.mobGrid[x][y] = -1;
                } else if(h2 + 1 < h1 || h2 - 1 > h1) {
                    this.mobGrid[x][y] = -1;
                } else if(h3 + 1 < h1 || h3 - 1 > h1) {
                    this.mobGrid[x][y] = -1;
                } else if(h4 + 1 < h1 || h4 - 1 > h1) {
                    this.mobGrid[x][y] = -1;
                }
            }
        }
    };


    this.startSpawn = function() {
        for(var i = 0; i < this.spawners.length; i++) {
            this.spawners[i].start();
        }
    };


    this.getTilePos = function(x, y) {
        x *= this.TILE_W;
        x += this.TILE_MID_X;
        if(y % 2 == 1) {
            x += this.TILE_VAR_W;
        }
        y *= this.TILE_H;
        y += this.TILE_MID_Y;
        return { x : x, y : y };
    };


    this.occupy = function(x, y, h, id) {
        var mobGridX = x - this.MOB_GRID_OFFSET_X;
        mobGridX = Math.floor(mobGridX / this.MOB_GRID_TILE_W);
        mobGridX = limit(mobGridX, 0, this.MOB_GRID_W - 1);
        var mobGridY = y - this.MOB_GRID_OFFSET_Y;
        mobGridY = Math.floor(mobGridY / this.MOB_GRID_TILE_H);
        mobGridY = limit(mobGridY, 0, this.MOB_GRID_H - 1);
        if(this.mobGrid[mobGridX][mobGridY] != 0) {
            this.mobGrid[mobGridX][mobGridY] = 666666;
        } else {
            this.mobGrid[mobGridX][mobGridY] = id;
        }

        var tileY = Math.floor((y + 12) / this.TILE_H);
        tileY = limit(tileY, 0, this.H - 1);
        var slot = (h * 10) + limit(Math.floor(((y + 12) - (tileY * this.TILE_H)) / 4), 0, 9);
        this.mobDrawGrid[tileY][slot].push(id);

        var updateX = Math.floor(x / this.MOB_UPDATE_GRID_TILE_W);
        updateX = limit(updateX, 0, this.MOB_UPDATE_GRID_W - 1);
        var updateY = Math.floor(y / this.MOB_UPDATE_GRID_TILE_H);
        updateY = limit(updateY, 0, this.MOB_UPDATE_GRID_H - 1);
        var updateQueue = this.mobUpdateGrid[updateX][updateY];
        updateQueue.push(id);
    };


    this.free = function(x, y, h, id) {

        var mobGridX = x - this.MOB_GRID_OFFSET_X;
        mobGridX = Math.floor(mobGridX / this.MOB_GRID_TILE_W);
        mobGridX = limit(mobGridX, 0, this.MOB_GRID_W - 1);
        var mobGridY = y - this.MOB_GRID_OFFSET_Y;
        mobGridY = Math.floor(mobGridY / this.MOB_GRID_TILE_H);
        mobGridY = limit(mobGridY, 0, this.MOB_GRID_H - 1);
        if(this.mobGrid[mobGridX][mobGridY] != 666666) {
            this.mobGrid[mobGridX][mobGridY] = 0;
        }

        var tileY = Math.floor((y + 12) / this.TILE_H);
        tileY = limit(tileY, 0, this.H - 1);
        var slot = (h * 10) + limit(Math.floor(((y + 12) - (tileY * this.TILE_H)) / 4), 0, 9);
        var mobs = this.mobDrawGrid[tileY][slot];
        var i = mobs.indexOf(id);
        if(i > -1) {
            mobs.splice(i, 1);
        }

        var updateX = Math.floor(x / this.MOB_UPDATE_GRID_TILE_W);
        updateX = limit(updateX, 0, this.MOB_UPDATE_GRID_W - 1);
        var updateY = Math.floor(y / this.MOB_UPDATE_GRID_TILE_H);
        updateY = limit(updateY, 0, this.MOB_UPDATE_GRID_H - 1);
        var updateQueue = this.mobUpdateGrid[updateX][updateY];
        i = updateQueue.indexOf(id);
        if(i > -1) {
            updateQueue.splice(i, 1);
        }
    };


    this.stepOnMe = function(x, y) {
        var mobGridX = x - this.MOB_GRID_OFFSET_X;
        mobGridX = Math.floor(mobGridX / this.MOB_GRID_TILE_W);
        mobGridX = limit(mobGridX, 0, this.MOB_GRID_W - 1);
        var mobGridY = y - this.MOB_GRID_OFFSET_Y;
        mobGridY = Math.floor(mobGridY / this.MOB_GRID_TILE_H);
        mobGridY = limit(mobGridY, 0, this.MOB_GRID_H - 1);
        if(this.mobGrid[mobGridX][mobGridY] != 666666) {
            this.mobGrid[mobGridX][mobGridY] = 0;
        }
    };


    this.isPosWalkable = function(x, y) {
        x -= this.MOB_GRID_OFFSET_X;
        x = Math.floor(x / this.MOB_GRID_TILE_W);
        y -= this.MOB_GRID_OFFSET_Y;
        y = Math.floor(y / this.MOB_GRID_TILE_H);
        x = limit(x, 0, this.MOB_GRID_W - 1);
        y = limit(y, 0, this.MOB_GRID_H - 1);
        return this.mobGrid[x][y] == 0;
    };


    this.isMobTileWalkable = function(x, y) {
        x = limit(x, 0, this.MOB_GRID_W - 1);
        y = limit(y, 0, this.MOB_GRID_H - 1);
        return this.mobGrid[x][y] == 0;
    };


    this.getMobTile = function(x, y) {
        x -= this.MOB_GRID_OFFSET_X;
        x = Math.floor(x / this.MOB_GRID_TILE_W);
        y -= this.MOB_GRID_OFFSET_Y;
        y = Math.floor(y / this.MOB_GRID_TILE_H);
        x = limit(x, 0, this.MOB_GRID_W - 1);
        y = limit(y, 0, this.MOB_GRID_H - 1);
        return { x : x, y : y };
    };


    this.getMobTileCenter = function(tilePos) {
        var x = ((tilePos.x + 0.5) * this.MOB_GRID_TILE_W) + this.MOB_GRID_OFFSET_X;
        var y = ((tilePos.y + 0.5) * this.MOB_GRID_TILE_H) + this.MOB_GRID_OFFSET_Y;
        return { x : x, y : y };
    };


    this.coordToTile = function(x, y) {

        var tileX = Math.floor(x / this.TILE_W);
        var tileOffsetX = x - (tileX * this.TILE_W);

        var tileY = Math.floor(y / this.TILE_H);
        var tileOffsetY = y - (tileY * this.TILE_H);

        if(tileY % 2 == 1) {
            tileOffsetX -= this.TILE_VAR_W;
            if(tileOffsetX < 0) {
                tileOffsetX += this.TILE_W;
                tileX--;
            }
        }

        if(tileOffsetY < this.TILE_VAR_H) {

            if(tileOffsetX < this.TILE_VAR_W) {
                if((tileOffsetY * this.TILE_RATIO) + tileOffsetX < this.TILE_VAR_W) {
                    tileY--;
                    if(tileY % 2 == 1) {
                        tileX--;
                    }
                }
            } else {
                tileOffsetX = this.TILE_W - tileOffsetX;
                if((tileOffsetY * this.TILE_RATIO) + tileOffsetX < this.TILE_VAR_W) {
                    tileY--;
                    if(tileY % 2 == 0) {
                        tileX++;
                    }
                }
            }
        }
        if(tileX < 0) {
            tileX = 0;
        } else if(tileX >= this.W) {
            tileX = this.W - 1;
        }
        if(tileY < 0) {
            tileY = 0;
        } else if(tileY >= this.H) {
            tileY = this.H - 1;
        }
        return { x : tileX, y : tileY };
    };


    this.getPosElevation = function(x, y) {
        var tile = this.coordToTile(x, y);
        return this.t[tile.x][tile.y].elevation;
    };


    this.getTileElevation = function(tile) {
        var x = limit(tile.x, 0, this.W - 1);
        var y = limit(tile.y, 0, this.H - 1);
        return this.t[x][y].elevation;
    };


    this.distance = function(one, other) {
        return distance(one.x, one.y / 0.7, other.x, other.y / 0.7);
    };


    this.update = function() {

        if(this.spawnerCounter > 0) {
            this.spawnerUpdateIndex++;
            if(this.spawnerUpdateIndex >= this.spawnerCounter) {
                this.spawnerUpdateIndex = 0;
            }
            this.spawners[this.spawnerUpdateIndex].update();
        }

        var limits = camera.getLimits();
        limits.x1 = limit(Math.floor(limits.x1 / this.MOB_UPDATE_GRID_TILE_W) - 1, 0, this.MOB_UPDATE_GRID_W - 1);
        limits.x2 = limit(Math.ceil(limits.x2 / this.MOB_UPDATE_GRID_TILE_W) + 1, 0, this.MOB_UPDATE_GRID_W - 1);
        limits.y1 = limit(Math.floor(limits.y1 / this.MOB_UPDATE_GRID_TILE_H) - 1, 0, this.MOB_UPDATE_GRID_H - 1);
        limits.y2 = limit(Math.ceil(limits.y2 / this.MOB_UPDATE_GRID_TILE_H) + 1, 0, this.MOB_UPDATE_GRID_H - 1);
        var x;
        var y;
        var t;
        var i;
        var updateQueue;
        var mob;
        for(x = limits.x1; x < limits.x2; x++) {
            for(y = limits.y1; y < limits.y2; y++) {
                t = this.mobUpdateGrid[x][y];
                updateQueue = [];
                for(i = 0; i < t.length; i++) {
                    if(t[i] > 1) {
                        updateQueue.push(t[i]);
                    }
                }
                for(i = 0; i < updateQueue.length; i++) {
                    var mob = this.mobs[updateQueue[i]];
                    if(mob.remove) {
                        mob.kill();
                        delete this.mobs[updateQueue[i]];
                    } else {
                        mob.update();
                    }
                }
            }
        }

        this.updateMapCountdown--;
        if(this.updateMapCountdown <= 0) {
            this.updateMapCountdown = 25;

            var playerTile = this.coordToTile(player.x, player.y);

            var limits = camera.getLimits();
            limits.x1 = limit(Math.floor(limits.x1 / this.TILE_W), 0, this.W - 1);
            limits.x2 = limit(Math.ceil(limits.x2 / this.TILE_W), 0, this.W - 1);
            limits.y1 = limit(Math.floor(limits.y1 / this.TILE_H), 0, this.H - 1);
            limits.y2 = limit(Math.ceil(limits.y2 / this.TILE_H), 0, this.H - 1);

            for(y = limits.y1; y < limits.y2; y++) {
                for(x = limits.x1; x < limits.x2; x++) {
                    if(distance(x, y, playerTile.x, playerTile.y) < 12) {
                        this.t[x][y].seen = true;
                    }
                }
            }
            if(miniMap.showing) {
                miniMap.generateMiniMap();
            }
        }

        this.liquidAni += timer.delta * 0.5;
        if(this.liquidAni > 1.0) {
            this.liquidAni -= 1.0;
        }
    };


    this.draw = function() {

        var y;
        var x;
        var ele;

        var t;
        var tilePos;
        var tX;
        var tY;

        var mobs;
        var mobSlot;

        var ani;
        var texOffset;

        var limits = camera.getLimits();
        limits.x1 = limit(Math.floor(limits.x1 / this.TILE_W) - 2, 0, this.W - 1);
        limits.x2 = limit(Math.ceil(limits.x2 / this.TILE_W) + 2, 0, this.W - 1);
        limits.y1 = limit(Math.floor(limits.y1 / this.TILE_H) - 2, 0, this.H - 1);
        limits.y2 = limit(Math.ceil(limits.y2 / this.TILE_H) + 4, 0, this.H - 1);

        for(y = limits.y1; y < limits.y2; y++) {
            for(x = limits.x1; x < limits.x2; x++) {
                t = this.t[x][y];
                if(t.liquid != 0) {
                    ani = this.liquidAni + t.liquidAniOffset;
                    texOffset = Math.floor(ani * 8);
                    tilePos = this.getTilePos(x, y);
                    tX = tilePos.x - 21;
                    tY = tilePos.y + Math.sin(TWO_PI * ani) - 10;
                    img.drawSprite("liquid", tX, tY, 46, 80, texOffset % 8, t.liquid - 1);
                    c.globalAlpha = (ani * 8) - texOffset;
                    img.drawSprite("liquid", tX, tY, 46, 80, (texOffset + 1) % 8, t.liquid - 1);
                    c.globalAlpha = 1.0;
                }
            }
            for(ele = 0; ele <= 7; ele++) {
                if(ele <= 6) {
                    for(x = limits.x1; x < limits.x2; x++) {
                        t = this.t[x][y];
                        if(t.elevation == ele) {
                            tilePos = this.getTilePos(x, y);
                            tX = tilePos.x - 21;
                            tY = tilePos.y - ((6 * ele) + 20);
                            if(t.liquid == 0) {
                                img.drawSprite("tiles", tX, tY, 46, 100, t.tileX, t.tileY);
                                if(t.path > 0) {
                                    img.drawSprite("path", tX, tY + 4, 46, 40, t.path, 0);
                                }
                                if(t.border != 0) {
                                    img.drawSprite("border", tX, tY + 4, 46, 40, t.border, 0);
                                }
                            }
                        }
                    }
                    mobs = this.mobDrawGrid[y];
                    mobSlot = (ele * 10);
                    for(var i = mobSlot; i < mobSlot + 10; i++) {
                        if(i < mobSlot + 3) {
                            for(var j = 0; j < mobs[i].length; j++) {
                                this.drawMobLegs(mobs[i][j]);
                                this.drawMob(mobs[i][j]);
                            }
                        } else {
                            for(var j = 0; j < mobs[i].length; j++) {
                                this.drawMobLegs(mobs[i][j]);
                            }
                        }
                    }
                    for(x = limits.x1; x < limits.x2; x++) {
                        t = this.t[x][y];
                        if(t.elevation == ele) {
                            tilePos = this.getTilePos(x, y);
                            tX = tilePos.x - 21;
                            tY = tilePos.y - ((6 * ele) + 20);
                            if(t.liquid == 0) {
                                if(t.object != -1) {
                                    var texX = t.object % 4;
                                    var texY = Math.floor(t.object / 4);
                                    img.drawSprite("objects", tX - 2, tY - 156, 50, 200, texX, texY);
                                }
                            }
                        }
                    }
                }
                if(ele >= 1) {
                    mobs = this.mobDrawGrid[y];
                    mobSlot = ((ele - 1) * 10);
                    for(var i = mobSlot; i < mobSlot + 10; i++) {
                        if(i >= mobSlot + 3) {
                            for(var j = 0; j < mobs[i].length; j++) {
                                this.drawMob(mobs[i][j]);
                                this.drawStatus(mobs[i][j]);
                            }
                        } else {
                            for(var j = 0; j < mobs[i].length; j++) {
                                this.drawStatus(mobs[i][j]);
                            }
                        }
                    }
                }
            }
        }

        //this.drawDebug();
    };


    this.drawMobLegs = function(id) {
        if(id == 1) {
            player.drawLegs();
        } else {
            this.mobs[id].drawLegs();
        }
    };


    this.drawMob = function(id) {
        if(id == 1) {
            player.draw();
        } else {
            this.mobs[id].draw();
        }
    };


    this.drawStatus = function(id) {
        if(id == 1) {
            player.drawStatus();
        } else {
            this.mobs[id].drawStatus();
        }
    };


    this.drawDebug = function() {

        var limits = camera.getLimits();
        limits.x1 = limit(Math.floor(limits.x1 / this.MOB_GRID_TILE_W) - 2, 0, this.MOB_GRID_W - 1);
        limits.x2 = limit(Math.ceil(limits.x2 / this.MOB_GRID_TILE_W) + 2, 0, this.MOB_GRID_W - 1);
        limits.y1 = limit(Math.floor(limits.y1 / this.MOB_GRID_TILE_H) - 2, 0, this.MOB_GRID_H - 1);
        limits.y2 = limit(Math.ceil(limits.y2 / this.MOB_GRID_TILE_H) + 2, 0, this.MOB_GRID_H - 1);

        c.save();
        c.translate(this.MOB_GRID_OFFSET_X, this.MOB_GRID_OFFSET_Y);
        c.globalAlpha = 0.6;
        for(var x = limits.x1; x <= limits.x2; x++) {
            for(var y = limits.y1; y <= limits.y2; y++) {
                if(this.mobGrid[x][y] == 0) {
                    c.fillStyle = "#0f0";
                } else if(this.mobGrid[x][y] == 666666) {
                    c.fillStyle = "#00f";
                } else {
                    c.fillStyle = "#f00";
                }
                c.fillRect((x * this.MOB_GRID_TILE_W) + 1, (y * this.MOB_GRID_TILE_H) + 1, this.MOB_GRID_TILE_W - 2, this.MOB_GRID_TILE_H - 2);
            }
        }
        c.globalAlpha = 1.0;
        c.restore();
    };

};