function Mob() {

    this.CHECK_FREQUENCY = 25;
    this.VIEW_RANGE = 200;

    this.typeName = "";
    this.type = null;

    this.id;

    this.side;

    this.parentSpawner;

    this.x = 0.0;
    this.y = 0.0;
    this.h = 0;

    this.tileX = 0.0;
    this.tileY = 0.0;

    this.speed = 80.0;

    this.figure = null;

    this.walkAni = 0.0;
    this.walkAmp = 1.0;
    this.direction = 0;

    this.checkCountdown = 0;
    this.path = [];

    this.aggro = false;

    this.life = 0;
    this.maxLife = 0;
    this.defence = 1.0;

    this.dead = false;
    this.deathAnimation = 0;
    this.remove = false;

    this.dotCountdown = 0.0;
    this.dotDamage = 0.0;
    this.stunCountdown = 0.0;

    this.attackedCooldown = 0.0;


    this.init = function(type, id, x, y, side, parentSpawner) {

        this.typeName = type;
        this.type = Mob.types[type];

        this.id = id;

        this.x = x;
        this.y = y;

        var mobTile = map.getMobTile(this.x, this.y);
        var tileCenter = map.getMobTileCenter(mobTile);

        this.tileX = this.x - tileCenter.x;
        this.tileY = this.y - tileCenter.y;
        this.x = tileCenter.x;
        this.y = tileCenter.y;

        this.side = side;

        this.parentSpawner = parentSpawner;

        var parts = this.type.generate();
        this.figure = new Figure();
        this.figure.init(this.side, parts.head, parts.body, parts.arms, parts.legs);
        this.speed = this.figure.getSpeed();
        this.life = this.figure.getLife();
        this.maxLife = this.life;
        this.defence = this.figure.getDefence();


        this.direction = rand(0, 7);

        this.checkCountdown = rand(1, this.CHECK_FREQUENCY);

        this.h = map.getPosElevation(this.x, this.y);

        map.occupy(this.x, this.y, this.h, this.id);
    };


    this.update = function() {

        if(!this.dead) {

            var canWalk = true;

            if(this.aggro) {
                canWalk = this.tryToAttack();
            }
            if(canWalk) {
                if(this.path.length > 0) {
                    if(this.aggro) {
                        this.checkCountdown--;
                        if(this.checkCountdown <= 0) {
                            this.checkCountdown = this.CHECK_FREQUENCY;
                            this.checkWhatToDo();
                        }
                        if(this.path.length > 0) {
                            this.walkPath();
                        }
                    } else {
                        this.walkPath();
                    }

                } else {
                    this.walkAmp = 0.0;
                    this.checkCountdown--;
                    if(this.checkCountdown <= 0) {
                        this.checkCountdown = this.CHECK_FREQUENCY;
                        this.checkWhatToDo();
                    }
                }
            }

            if(this.dotCountdown > 0.0) {
                this.dotCountdown -= timer.delta;
                this.life -= this.dotDamage * timer.delta;
                this.testForDeath();
                if(this.dotCountdown < 0.0) {
                    this.dotCountdown = 0.0;
                }
            }

            if(this.stunCountdown > 0.0) {
                this.stunCountdown -= timer.delta;
                if(this.stunCountdown < 0.0) {
                    this.stunCountdown = 0.0;
                }
            }

            if(this.attackedCooldown > 0.0) {
                this.attackedCooldown -= timer.delta;
                if(this.attackedCooldown < 0.0) {
                    this.attackedCooldown = 0.0;
                }
            }

        } else {
            this.deathAnimation += timer.delta;
            if(this.deathAnimation >= 10.0) {
                this.remove = true;
            }
        }
        this.figure.update(this.stunCountdown);
    };


    this.tryToAttack = function() {
        var updateX = Math.floor(this.x / map.MOB_UPDATE_GRID_TILE_W);
        var startX = limit(updateX - 1, 0, map.MOB_UPDATE_GRID_W - 1);
        var endX = limit(updateX + 1, 0, map.MOB_UPDATE_GRID_W - 1);

        var updateY = Math.floor(this.y / map.MOB_UPDATE_GRID_TILE_H);
        var startY = limit(updateY - 1, 0, map.MOB_UPDATE_GRID_H - 1);
        var endY = limit(updateY + 1, 0, map.MOB_UPDATE_GRID_H - 1);

        for(var hitX = startX; hitX <= endX; hitX++) {
            for(var hitY = startY; hitY <= endY; hitY++) {
                var ts = map.mobUpdateGrid[hitX][hitY];
                for(var i = 0; i < ts.length; i++) {
                    var t;
                    if(ts[i] > 1) {
                        t = map.mobs[ts[i]];
                    } else {
                        t = player;
                    }
                    if(t.side != this.side && !t.dead && (t.side == 1 || ts[i] > 1 || this.typeName != player.figure.head.type || this.attackedCooldown > 0.0)) {
                        var deltaX = t.x - this.x;
                        var deltaY = t.y - this.y;
                        if(Math.sqrt((deltaX * deltaX) + (deltaY * deltaY * 0.64)) < this.figure.arms.attackRange * 0.6) {
                            var angle = Math.atan2(deltaX, -deltaY) + (2.125 * PI);
                            this.direction =  Math.round(angle / (0.25 * PI)) % 8;
                            if(this.figure.canMainAttack()) {
                                this.figure.mainAttack(this.x, this.y, this.direction);
                            }
                            return false;
                        }
                    }
                }
            }
        }
        return true;
    };


    this.checkWhatToDo = function() {
        var foundTarget = false;
        var updateX = Math.floor(this.x / map.MOB_UPDATE_GRID_TILE_W);
        var startX = limit(updateX - 2, 0, map.MOB_UPDATE_GRID_W - 1);
        var endX = limit(updateX + 2, 0, map.MOB_UPDATE_GRID_W - 1);

        var updateY = Math.floor(this.y / map.MOB_UPDATE_GRID_TILE_H);
        var startY = limit(updateY - 2, 0, map.MOB_UPDATE_GRID_H - 1);
        var endY = limit(updateY + 2, 0, map.MOB_UPDATE_GRID_H - 1);

        for(var hitX = startX; hitX <= endX; hitX++) {
            for(var hitY = startY; hitY <= endY; hitY++) {
                var ts = map.mobUpdateGrid[hitX][hitY];
                for(var i = 0; i < ts.length; i++) {
                    var t;
                    if(ts[i] > 1) {
                        t = map.mobs[ts[i]];
                    } else {
                        t = player;
                    }
                    if(t.side != this.side && !t.dead && (t.side == 1 || ts[i] > 1 || this.typeName != player.figure.head.type || this.attackedCooldown > 0.0)) {
                        var deltaX = t.x - this.x;
                        var deltaY = t.y - this.y;
                        if(Math.sqrt((deltaX * deltaX) + (deltaY * deltaY * 0.64)) < this.VIEW_RANGE) {
                            this.aggro = true;
                            this.walkToEnemy(t);
                            foundTarget = true;
                        }
                    }
                }
            }
        }
        if(!foundTarget) {
            this.aggro = false;
            this.findRandomTarget();
        }
    };


    this.walkToEnemy = function(t) {
        var mobTile = map.getMobTile(t.x, t.y);
        var gotPath = false;
        var radius = this.figure.arms.attackRange / 20;
        var tries = 0;
        while(!gotPath && tries < 20) {
            var randAngle = randFloat(0, TWO_PI);
            var r = radius + (tries / 2);
            var x = Math.round(mobTile.x + (Math.sin(randAngle) * r));
            var y = Math.round(mobTile.y + (Math.cos(randAngle) * r));
            if(map.isMobTileWalkable(x, y)) {
                this.findPathTo(x, y);
                gotPath = true;
            }
            tries++;
        }
    };


    this.kill = function() {
        map.spawners[this.parentSpawner].hasMobs--;
        map.free(this.x, this.y, this.h, this.id);
    };


    this.hit = function(damage, x, y) {
        if(!this.dead) {
            if(player.tutorial == 1) {
                player.tutorial = 2;
                panel.hideTutorial();
            } else if(player.tutorial == 3) {
                player.tutorial = 4;
                panel.hideTutorial();
            }
            var factor = 1.0 - (this.defence / 10);
            this.life -= damage * factor;
            this.figure.hit(x, y);
            this.testForDeath();
            this.attackedCooldown = 20.0;
        }
    };


    this.testForDeath = function() {
        if(this.life <= 0) {
            this.life = 0;
            this.dead = true;
            map.stepOnMe(this.x, this.y);
            if(this.side == 1) {
                player.points += this.type.points;
                if(this.typeName == "lindworm" || rand(0, 3) == 0) {

                    var tries = 50;
                    var foundBodyPart = false;
                    while(tries > 0 && !foundBodyPart) {

                        var bodyPart = rand(0, 3);

                        if(bodyPart == 0) {
                            if(player.figure.head.type != this.figure.head.type) {
                                panel.choiceLoc = "head";
                                panel.choicePart = this.figure.headName;
                                foundBodyPart = true;
                            }

                        } else if(bodyPart == 1) {
                            if(player.figure.body.type != this.figure.body.type) {
                                panel.choiceLoc = "body";
                                panel.choicePart = this.figure.bodyName;
                                foundBodyPart = true;
                            }

                        } else if(bodyPart == 2) {
                            if(player.figure.arms.type != this.figure.arms.type) {
                                panel.choiceLoc = "arms";
                                panel.choicePart = this.figure.armsName;
                                foundBodyPart = true;
                            }

                        } else if(bodyPart == 3) {
                            if(player.figure.legs.type != this.figure.legs.type) {
                                panel.choiceLoc = "legs";
                                panel.choicePart = this.figure.legsName;
                                foundBodyPart = true;
                            }
                        }
                        if(foundBodyPart) {
                            panel.noticeLocationX = this.x + camera.OFFSET_X - camera.x;
                            panel.noticeLocationY = this.y + camera.OFFSET_Y - camera.y;
                            panel.showNotice();
                        }
                        tries--;
                    }
                }
            }
        }
    };


    this.knockback = function(angle, radius) {
        map.free(this.x, this.y, this.h, this.id);
        radius *= 15;
        var pushToR = 0;
        var pushByX = Math.sin(angle) * 5.0;
        var pushByY = -Math.cos(angle) * 3.5;
        while(map.isPosWalkable(this.x + pushByX, this.y + pushByY) && pushToR < radius) {
            this.x += pushByX;
            this.y += pushByY;
            pushToR += 5.01;
        }
        map.occupy(this.x, this.y, this.h, this.id);
    };


    this.walkPath = function() {

        map.free(this.x, this.y, this.h, this.id);

        var lastX = this.x;
        var lastY = this.y;

        var diffX = this.path[0].x - this.x;
        var diffY = this.path[0].y - this.y;

        var moveVector = normalize({ x : diffX, y : diffY });

        var tempVelocity = this.speed;
        if(this.stunCountdown > 0.0) {
            tempVelocity *= 0.2;
        }

        var moveX = moveVector.x * tempVelocity * timer.delta;
        var moveY = moveVector.y * tempVelocity * timer.delta;

        if(Math.abs(diffX) > Math.abs(moveX) || Math.abs(diffY) > Math.abs(moveY)) {
            var newX = this.x + moveX;
            var newY = this.y + moveY;
            if(map.isPosWalkable(newX, newY)) {
                this.x = newX;
                this.y = newY;
            } else {
                this.path = [];
            }

        } else {
            this.x = this.path[0].x;
            this.y = this.path[0].y;
            this.path.splice(0, 1);
        }

        if(this.x == lastX && this.y < lastY) {
            this.direction = 0;
        } else if(this.x > lastX && this.y < lastY) {
            this.direction = 1;
        } else if(this.x > lastX && this.y == lastY) {
            this.direction = 2;
        } else if(this.x > lastX && this.y > lastY) {
            this.direction = 3;
        } else if(this.x == lastX && this.y > lastY) {
            this.direction = 4;
        } else if(this.x < lastX && this.y > lastY) {
            this.direction = 5;
        } else if(this.x < lastX && this.y == lastY) {
            this.direction = 6;
        } else if(this.x < lastX && this.y < lastY) {
            this.direction = 7;
        }

        this.walkAmp = 1.0;
        if(this.figure.legsName == "lindworm") {
            this.walkAni += 0.015 * tempVelocity * timer.delta;
        } else {
            this.walkAni += 0.04 * tempVelocity * timer.delta;
        }
        while(this.walkAni > 1.0) {
            this.walkAni -= 1.0
        }
        this.h = map.getPosElevation(this.x, this.y);

        map.occupy(this.x, this.y, this.h, this.id);
    };


    this.findRandomTarget = function() {
        if(zeroOrOne(0.03)) {
            var mobTile = map.getMobTile(this.x, this.y);
            var randX = mobTile.x - rand(-10, 10);
            var randY = mobTile.y - rand(-10, 10);
            if(map.isMobTileWalkable(randX, randY)) {
                this.findPathTo(randX, randY);
            }
        }
    };


    this.findPathTo = function(endX, endY) {
        var mobTile = map.getMobTile(this.x, this.y);
        var start = { x : mobTile.x, y : mobTile.y };
        var end = { x : Math.round(endX), y : Math.round(endY) };
        var pathFinding = new PathFinding();
        var path = pathFinding.find(start, end, 8, function(x, y) {
            return map.isMobTileWalkable(x, y);
        });
        this.path = [];
        for(var i = 0; i < path.length; i++) {
            this.path.push(map.getMobTileCenter(path[i]));
        }
        if(this.path.length > 0) {
            this.checkCountdown = this.CHECK_FREQUENCY;
        }
    };


    this.drawLegs = function() {
        if(this.deathAnimation > 0.0) {
            c.globalAlpha = limit(1.0 - ((this.deathAnimation - 6.0) / 4.0), 0.0, 1.0);
        }
        c.save();
        c.translate(Math.round(this.x + this.tileX), Math.round(this.y + this.tileY - (this.h * 8)));
        this.figure.drawLegs(this.direction, this.walkAni, this.walkAmp);
        c.restore();
        if(this.deathAnimation > 0.0) {
            c.globalAlpha = 1.0;
        }
    };


    this.draw = function() {
        if(this.deathAnimation > 0.0) {
            c.globalAlpha = limit(1.0 - ((this.deathAnimation - 6.0) / 4.0), 0.0, 1.0);
        }
        c.save();
        c.translate(Math.round(this.x + this.tileX), Math.round(this.y + this.tileY - (this.h * 8)));
        this.figure.draw(this.direction, this.walkAni, this.walkAmp, this.deathAnimation);
        c.restore();
        if(this.deathAnimation > 0.0) {
            c.globalAlpha = 1.0;
        }
    };


    this.drawStatus = function() {
        if(this.deathAnimation > 0.0) {
            c.globalAlpha = limit(1.0 - ((this.deathAnimation - 6.0) / 4.0), 0.0, 1.0);
        }
        c.save();
        c.translate(Math.round(this.x + this.tileX), Math.round(this.y + this.tileY - (this.h * 8)));
        this.figure.drawStatus(true, this.life, this.maxLife, this.dotCountdown, this.stunCountdown);
        c.restore();
        if(this.deathAnimation > 0.0) {
            c.globalAlpha = 1.0;
        }
    };

}



Mob.types = {

    human : {
        name : "Human",
        names : "Humans",
        points : 100,
        generate : function() {
            var skin = Mob.skin[rand(0, 1)];
            return {
                head : "human_" + skin + "_" + Mob.gender[rand(0, 1)],
                body : "human_" + rand(0, 3),
                arms : "human_" + skin,
                legs : "human_" + rand(0, 1),
            };
        }
    },

    slime : {
        name : "Slime",
        names : "Slimes",
        points : 4400,
        generate : function() {
            var color = rand(0, 2);
            return {
                head : "slime_" + color,
                body : "slime_" + color,
                arms : "slime_" + color,
                legs : "slime_" + color,
            };
        }
    },

    zombie : {
        name : "Zombie",
        names : "Zombies",
        points : 3200,
        generate : function() {
            return {
                head : "zombie_" + rand(0, 1),
                body : "zombie_" + rand(0, 1),
                arms : "zombie",
                legs : "zombie",
            };
        }
    },

    skeleton : {
        name : "Skeleton",
        names : "Skeletons",
        points : 2500,
        generate : function() {
            return {
                head : "skeleton",
                body : "skeleton",
                arms : "skeleton",
                legs : "skeleton",
            };
        }
    },

    lindworm : {
        name : "Lindworm",
        names : "Lindworms",
        points : 12000,
        generate : function() {
            return {
                head : "lindworm",
                body : "lindworm",
                arms : "lindworm",
                legs : "lindworm",
            };
        }
    },

    goblin : {
        name : "Goblin",
        names : "Goblins",
        points : 250,
        generate : function() {
            return {
                head : "goblin",
                body : "goblin",
                arms : "goblin",
                legs : "goblin",
            };
        }
    },

    yeti : {
        name : "Yeti",
        names : "Yetis",
        points : 500,
        generate : function() {
            return {
                head : "yeti",
                body : "yeti",
                arms : "yeti",
                legs : "yeti",
            };
        }
    },

    ent : {
        name : "Ent",
        names : "Ents",
        points : 800,
        generate : function() {
            return {
                head : "ent",
                body : "ent",
                arms : "ent",
                legs : "ent",
            };
        }
    },

    walker : {
        name : "Walker",
        names : "Walkers",
        points : 1200,
        generate : function() {
            return {
                head : "walker",
                body : "walker",
                arms : "walker",
                legs : "walker",
            };
        }
    },

    golem : {
        name : "Golem",
        names : "Golems",
        points : 3000,
        generate : function() {
            return {
                head : "golem",
                body : "golem",
                arms : "golem",
                legs : "golem",
            };
        }
    },

    demon : {
        name : "Demon",
        names : "Demons",
        points : 6400,
        generate : function() {
            return {
                head : "demon",
                body : "demon",
                arms : "demon",
                legs : "demon",
            };
        }
    },

    minotaur : {
        name : "Minotaur",
        names : "Minotaurs",
        points : 4800,
        generate : function() {
            return {
                head : "minotaur",
                body : "minotaur",
                arms : "minotaur",
                legs : "minotaur",
            };
        }
    },
};


Mob.skin = ["w", "b"];
Mob.gender = ["m", "f"];