function Player() {

    this.NO_COLLISION_DEBUG = false;
    this.SPAWN_TILE = { x : 40, y : 50 };

    this.side = 0;

    this.x = 0;
    this.y = 0;
    this.h = 0;

    this.vector = { x : 0, y : 0 };

    this.speed = 120.0;
    this.acceleration = 500.0;

    this.figure = null;

    this.walkAni = 0.0;
    this.walkAmp = 0.0;
    this.velocity = 0.0;
    this.direction = 4;

    this.life = 100;
    this.maxLife = 100;
    this.defence = 0;

    this.dead = false;
    this.deathAnimation = 0.0;

    this.points = 0;

    this.dotCountdown = 0.0;
    this.dotDamage = 0.0;
    this.stunCountdown = 0.0;

    this.healingCountdown = 0.0;

    this.won = false;

    this.tutorial = 0;


    this.init = function() {

        var tilePos = map.getTilePos(this.SPAWN_TILE.x, this.SPAWN_TILE.y);

        var foundSpawnPoint = false;
        while(!foundSpawnPoint) {
            var candidate = {
                x : tilePos.x + rand(-5, 5),
                y : tilePos.y + rand(-5, 5)
            };
            if(map.isPosWalkable(candidate.x, candidate.y)) {
                this.x = candidate.x;
                this.y = candidate.y;
                foundSpawnPoint = true;
            }
        }

        this.figure = new Figure();
        var gender = ["m", "f"];
        gender = gender[rand(0, 1)];
        var skin = ["w", "b"];
        skin = skin[rand(0, 1)];
        this.figure.init(0, "human_" + skin + "_" + gender, "human_" + rand(0, 3), "human_" + skin, "human_" + rand(0, 1));

        this.speed = this.figure.getSpeed() * 2.0;
        this.life = this.figure.getLife() * 25.0;
        this.maxLife = this.life;
        this.defence = this.figure.getDefence();

        this.h = map.getPosElevation(this.x, this.y);
        map.occupy(this.x, this.y, this.h, 1);

        keyboard.registerKeyDownHandler(Keyboard.SPACE_BAR, function() {
            player.attack(0);
        });
        keyboard.registerKeyDownHandler(Keyboard.X, function() {
            player.attack(1);
        });
    };


    this.setBodyPart = function(choiceLoc, choicePart) {
        this.figure.setPart(choiceLoc, choicePart);
        this.speed = this.figure.getSpeed() * 2.0;
        this.life = this.figure.getLife() * 100.0;
        this.maxLife = this.life;
        this.defence = this.figure.getDefence();
        if(this.figure.headName == "lindworm" &&
            this.figure.bodyName == "lindworm" &&
            this.figure.armsName == "lindworm" &&
            this.figure.legsName == "lindworm") {
            this.won = true;
        }
    }


    this.attack = function(type) {
        if(type == 0) {
            if(this.figure.canMainAttack()) {
                panel.spaceCountdown = 0.5;
                this.figure.mainAttack(this.x, this.y, this.direction);
            }
        }
        if(type == 1) {
            if(this.figure.hasSecondAttack() && this.figure.canSecondAttack()) {
                panel.xCountdown = 0.5;
                this.figure.secondAttack(this.x, this.y, this.direction);
            }
        }
    };


    this.hit = function(damage, x, y) {
        if(!this.dead) {
            var factor = 1.0 - (this.defence / 10);
            this.life -= damage * factor;
            this.figure.hit(x, y);
            this.testForDeath();
            this.healingCountdown = 4.0;
        }
    };


    this.testForDeath = function() {
        if(this.life <= 0) {
            this.life = 0;
            this.dead = true;
            game.paused = true;
            jQuery("#gameover").html("Score:<br />" + this.points).fadeIn(3000);
            keyboard.registerKeyUpHandler(Keyboard.ENTER, function() {
                keyboard.deleteKeyUpHandler(Keyboard.ENTER);
                game.state.hide();
                game.state.show();
                jQuery("#gameover").hide();
                game.paused = false;
            })
        }
    };


    this.knockback = function(angle, radius) {
        map.free(this.x, this.y, this.h, 1);
        radius *= 15;
        var pushToR = 0;
        var pushByX = Math.sin(angle) * 5.0;
        var pushByY = -Math.cos(angle) * 3.5;
        while(map.isPosWalkable(this.x + pushByX, this.y + pushByY) && pushToR < radius) {
            this.x += pushByX;
            this.y += pushByY;
            pushToR += 5.01;
        }
        map.occupy(this.x, this.y, this.h, 1);
    };


    this.update = function() {

        if(keyboard.isPressed(Keyboard.SPACE_BAR)) {
            player.attack(0);
        }
        if(keyboard.isPressed(Keyboard.X)) {
            player.attack(1);
        }

        map.free(this.x, this.y, this.h, 1);

        var lastX = this.x;
        var lastY = this.y;

        var moveUpPressed = keyboard.isPressed(Keyboard.ARROW_UP) || keyboard.isPressed(Keyboard.W);
        var moveRightPressed = keyboard.isPressed(Keyboard.ARROW_RIGHT) || keyboard.isPressed(Keyboard.D);
        var moveDownPressed = keyboard.isPressed(Keyboard.ARROW_DOWN) || keyboard.isPressed(Keyboard.S);
        var moveLeftPressed = keyboard.isPressed(Keyboard.ARROW_LEFT) || keyboard.isPressed(Keyboard.A);

        var accelerate = false;
        var newVector = { x : 0.0, y : 0.0 };

        if(moveUpPressed && !moveDownPressed) {
            newVector.y -= 1.0;
            accelerate = true;
        } else if(moveDownPressed && !moveUpPressed) {
            newVector.y += 1.0;
            accelerate = true;
        }
        if(moveRightPressed && !moveLeftPressed) {
            newVector.x += 1.0;
            accelerate = true;
        } else if(moveLeftPressed && !moveRightPressed) {
            newVector.x -= 1.0;
            accelerate = true;
        }

        newVector = normalize(newVector);

        if(newVector.x != 0.0 || newVector.y != 0.0) {
            this.vector = newVector;
        }

        if(accelerate) {
            if(this.velocity < this.speed) {
                this.velocity += this.acceleration * timer.delta;
                if(this.velocity > this.speed) {
                    this.velocity = this.speed;
                }
            }
        } else {
            if(this.velocity > 0.0) {
                this.velocity -= this.acceleration * timer.delta;
                if(this.velocity < 0.0) {
                    this.velocity = 0.0;
                    this.walkAni = 0.0;
                }
            }
        }

        var tempVelocity = this.velocity;
        if(this.stunCountdown > 0.0) {
            tempVelocity *= 0.2;
        }

        var newX = this.x + (this.vector.x * tempVelocity * timer.delta);
        var newY = this.y + (this.vector.y * tempVelocity * timer.delta * 0.7);

        if(map.isPosWalkable(newX, newY) || this.NO_COLLISION_DEBUG) {
            this.x = newX;
            this.y = newY;
        } else {
            if(this.vector.x != 0 && map.isPosWalkable(newX, this.y)) {
                this.x = newX;
            } else if(this.vector.y != 0 && map.isPosWalkable(this.x, newY)) {
                this.y = newY;
            } else {
                this.velocity = 0;
                this.walkAni = 0.0;
            }
        }

        if(this.x == lastX && this.y < lastY) {
            this.direction = 0;
        } else if(this.x > lastX && this.y < lastY) {
            this.direction = 1;
        } else if(this.x > lastX && this.y == lastY) {
            this.direction = 2;
        } else if(this.x > lastX && this.y > lastY) {
            this.direction = 3;
        } else if(this.x == lastX && this.y > lastY) {
            this.direction = 4;
        } else if(this.x < lastX && this.y > lastY) {
            this.direction = 5;
        } else if(this.x < lastX && this.y == lastY) {
            this.direction = 6;
        } else if(this.x < lastX && this.y < lastY) {
            this.direction = 7;
        }

        if(this.x != lastX || this.y != lastY) {
            if(this.figure.legsName == "lindworm") {
                this.walkAni += 0.015 * tempVelocity * timer.delta;
            } else {
                this.walkAni += 0.02 * tempVelocity * timer.delta;
            }
            while(this.walkAni > 1.0) {
                this.walkAni -= 1.0
            }
            this.h = map.getPosElevation(this.x, this.y);
        }
        this.walkAmp = tempVelocity / this.speed;

        map.occupy(this.x, this.y, this.h, 1);

        if(this.dotCountdown > 0.0) {
            this.dotCountdown -= timer.delta;
            this.life -= this.dotDamage * timer.delta;
            this.testForDeath();
            if(this.dotCountdown < 0.0) {
                this.dotCountdown = 0.0;
            }
        }

        if(this.stunCountdown > 0.0) {
            this.stunCountdown -= timer.delta;
            if(this.stunCountdown < 0.0) {
                this.stunCountdown = 0.0;
            }
        }

        if(this.healingCountdown > 0.0) {
            this.healingCountdown -= timer.delta;
            if(this.healingCountdown < 0.0) {
                this.healingCountdown = 0.0;
            }
        } else {
            if(this.life < this.maxLife) {
                this.life += (this.maxLife * 0.2) * timer.delta;
                if(this.tutorial == 2) {
                    this.tutorial = 3;
                    panel.showTutorial("You&nbsp; regain&nbsp; health&nbsp; when&nbsp; you&nbsp; are&nbsp; out&nbsp; of&nbsp; battle&nbsp; for&nbsp; a&nbsp; few&nbsp; seconds.");
                } else if(this.tutorial == 4) {
                    this.tutorial = 5;
                    panel.showTutorial("Hold&nbsp; the&nbsp; m&nbsp; key&nbsp; to&nbsp; open&nbsp; the&nbsp; overview&nbsp; map.");
                }
                if(this.life > this.maxLife) {
                    this.life = this.maxLife;
                }
            }
        }

        this.figure.update(this.stunCountdown);
    };


    this.drawLegs = function() {
        c.save();
        c.translate(Math.round(this.x), Math.round(this.y - (this.h * 8)));
        this.figure.drawLegs(this.direction, this.walkAni, this.walkAmp);
        c.restore();
    };


    this.draw = function() {
        c.save();
        c.translate(Math.round(this.x), Math.round(this.y - (this.h * 8)));
        this.figure.draw(this.direction, this.walkAni, this.walkAmp, this.deathAnimation);
        c.restore();
    };


    this.drawStatus = function() {
        c.save();
        c.translate(Math.round(this.x), Math.round(this.y - (this.h * 8)));
        this.figure.drawStatus(false, this.life, 0, this.dotCountdown, this.stunCountdown);
        c.restore();
    };

};